
//
// Copyright 1994 (c) Todd Anthony Nathan.  All Rights Reserved.
//
// Wed Apr 27 21:29:45 GMT-0700 1994
//

#import <appkit/appkit.h>
#import "ProgressPanel.h"
#import "ProgressPanelConstants.h"
#import "MiscProgressView.h"
#import "MiscProgressBar.h"
#import "MiscProgressPie.h"	// Will use this later when we give caller option to specify
							//  pie or bar for the progress view type.
#import <appkit/nextstd.h>		// NXLogError
#import <mach/cthreads.h>

/*
 * Define the list of commands and functions this XModule implements
 */



/*
 * Keep track of the mach header where we get loaded
 * so we can use it to load other stuff later (such as
 * nib files)
 * Note: only needed in XModules that load nib files, etc.
 */
 
static struct mach_header	*loadHeader = NULL;

@implementation ProgressPanel : XModule

- init
{
	[super init];

	return ( self );
}

- free
{
	// Do your stuff here...

	return ( [super free] );
}

+ (const char *)moduleName
{
	return ( "ProgressPanel" );
}

+ (const char **)xCmdsAndFcns
{
	static char *handlerList[] = {
		"clearprogressview",
		"hideprogresspanel",
		"incrementprogressview",
		"progresspanel",
		"setprogressviewemphasis",
		"setprogressviewincrement",
		"showprogresspanel",
		"pp_version",
		NULL
	};
	return ( handlerList );
}

- prepareToExecuteHandlers
{
	// [self loadInterfaceData];
	return ( self );
}

- executeHandler:(NXAtom)handlerName
{
	if (!strcmp(handlerName, "progresspanel")) {
		return ( [self progressPanel] );
	} else if (!strcmp(handlerName, "clearprogressview")) {
		return ( [self clearProgressView] );
	} else if (!strcmp(handlerName, "hideprogresspanel")) {
		return ( [self hideProgressPanel] );
	} else if (!strcmp(handlerName, "incrementprogressview")) {
		return ( [self incrementProgressView] );
	} else if (!strcmp(handlerName, "progresspanel")) {
		return ( [self progressPanel] );
	} else if (!strcmp(handlerName, "setprogressviewincrement")) {
		return ( [self setProgressViewIncrement] );
	} else if (!strcmp(handlerName, "showprogresspanel")) {
		return ( [self showProgressPanel] );
	} else if (!strcmp(handlerName, "setprogressviewemphasis")) {
		return ( [self setProgressViewEmphasis] );
	} else if (!strcmp(handlerName, "pp_version")) {
		return ( [self pp_version] );
	} else  return ( nil ); // handlerName not recognized!
}

+ finishLoading:(struct mach_header *)header
{
	loadHeader = header; // Save this for later use
	return ( self );
}

- loadInterfaceData
{
	//[NXApp loadNibSection:"MyXModule.nib" owner:self
	//    withNames:NO fromHeader:loadHeader] )
		
	return ( self );
}

// XModule specific code...
//
// Usage: progressPanel  <winTitle>, <information>, <hiValue>, <loValue>
- progressPanel
{
	int				pCount = [self paramCount];
	const char		*panelTitle, *info;
	int				loValue, hiValue;
	NXRect			contentRect, progressBarFrame;
	NXRect			loTextFieldFrame, hiTextFieldFrame, infoTextFieldFrame;
	//NXRect		pauseButtonFrame, cancelButtonFrame;

	// Check to make sure the user used the right number of params.
	if ( pCount != 4 ) {
		return ( [[self getEmptyContainer] setString:
			"Usage: progressPanel  <winTitle>, <information>, <hiValue>, <loValue>"] );
	}

	// Collect the pointers to the parameter string values.
	panelTitle = [[self getParam:1] stringVal];
	info = [[self getParam:2] stringVal];
	loValue = [[self getParam:3] intVal];
	hiValue = [[self getParam:4] intVal];

	// Check and see if the panel already exists, and if not create it with
	// the sizes and properties that are in the Sample.nib in the source directory.
	if ( !progressPanel ) {
		NXSetRect(&contentRect, 0.0, 0.0, 400.0, 69.0 );
		NXSetRect(&progressBarFrame, 13.0, 33.0, 371.0, 18.0 );
		NXSetRect(&loTextFieldFrame, 15.0, 8.0, 65.0, 15.0 );
		NXSetRect(&hiTextFieldFrame, 316.0, 8.0, 65.0, 15.0 );
		NXSetRect(&infoTextFieldFrame, 83.0, 8.0, 230.0, 15.0 );
		//NXSetRect(&pauseButtonFrame, 180.0, 11.0, 100.0, 24.0 );
		//NXSetRect(&cancelButtonFrame, 283.0, 11.0, 100.0, 24.0 );

		// Create the panel, and name it.
		progressPanel = [[Panel alloc] initContent:&contentRect
				style:NX_TITLEDSTYLE backing:NX_BUFFERED buttonMask:0 defer:YES];
		[[[progressPanel setTitle:panelTitle] setFloatingPanel:YES] setHideOnDeactivate:YES];

		// Now create and setup the MiscProgress[Bar|Pie]
		progressView = [[MiscProgressBar alloc] initFrame:&progressBarFrame];
		[progressView setTicsOverBar:YES];
		[progressView setTickColor:NX_COLORBLACK];  // defined as  NXConvertGrayToColor(0.0)
		[progressView setNumTicks:(hiValue - loValue) - 1];
		[progressView setEmphasis:5];

		// Now create and setup the text fields (lo, hi, and info)
		loTextField = [[TextField alloc] initFrame:&loTextFieldFrame];
		[[[[[[loTextField setEnabled:NO]
				setTextColor:NX_COLORBLACK]
				setBackgroundColor:NX_COLORLTGRAY]
				setBezeled:NO] setBordered:NO] setBackgroundTransparent:YES];
		[[loTextField setIntValue:loValue] setAlignment:NX_LEFTALIGNED];
		hiTextField = [[TextField alloc] initFrame:&hiTextFieldFrame];
		[[[[[[hiTextField setEnabled:NO]
				setTextColor:NX_COLORBLACK]
				setBackgroundColor:NX_COLORLTGRAY]
				setBezeled:NO] setBordered:NO] setBackgroundTransparent:YES];
		[[hiTextField setIntValue:hiValue] setAlignment:NX_RIGHTALIGNED];
		infoTextField = [[TextField alloc] initFrame:&infoTextFieldFrame];
		[[[[[[infoTextField setEnabled:NO]
				setTextColor:NX_COLORBLACK]
				setBackgroundColor:NX_COLORLTGRAY]
				setBezeled:NO] setBordered:NO] setBackgroundTransparent:YES];
		[[infoTextField setStringValue:info] setAlignment:NX_CENTERED];

		// Create the pause and cancel buttons, making sure to 'connect' for action messages.
		//pauseButton = [[Button alloc] initFrame:&pauseButtonFrame
		//		title:"Pause" tag:0 target:self
		//		action:@selector(pauseProgress:) key:(unsigned short)0 enabled:YES];
		//[pauseButton setIconPosition:NX_TITLEONLY ];
		//cancelButton = [[Button alloc] initFrame:&cancelButtonFrame
		//		icon:"NXreturnSign" tag:0 target:self
		//		action:@selector(cancelProgress:) key:(unsigned short)0 enabled:YES];
		//[cancelButton setTitle:"Cancel"];
		//[[cancelButton setIconPosition:NX_ICONRIGHT] setAlignment:NX_CENTERED];

		// And finally add all the subviews to the content view of the panel
		[[progressPanel contentView] addSubview:progressView];
		[[progressPanel contentView] addSubview:loTextField];
		[[progressPanel contentView] addSubview:hiTextField];
		[[progressPanel contentView] addSubview:infoTextField];
		[[progressPanel contentView] addSubview:pauseButton];
		[[progressPanel contentView] addSubview:cancelButton];
	} else {
		[loTextField setIntValue:loValue];
		[hiTextField setIntValue:hiValue];
		[infoTextField setStringValue:info];
		[progressPanel setTitle:panelTitle];
		[[progressView setNumTicks:(hiValue - loValue) - 1] clear:self];
	}
	[[progressPanel orderFront:self] center];

	return ( [[self getEmptyContainer] setString:""] );
}

- setProgressViewIncrement
{
	int		pCount = [self paramCount];
	int		oldStepSize;

	if ( pCount != 1 || !([[self getParam:1] validAsNumber]) ) {
		return ( [[self getEmptyContainer] setString:
			"Usage: setProgressViewIncrement  <intValue>"] );
	}

	oldStepSize = [progressView stepSize];
	[progressView setStepSize:[[self getParam:1] intVal]];

	return ( [[self getEmptyContainer] setInt:oldStepSize] );
}

- incrementProgressView
{
	[progressView increment:self];
	return ( [[self getEmptyContainer] setString:""] );
}

- hideProgressPanel
{
	[progressPanel orderOut:self];
	return ( [[self getEmptyContainer] setString:""] );
}

- showProgressPanel
{
	[progressPanel orderFront:self];
	return ( [[self getEmptyContainer] setString:""] );
}

- clearProgressView
{
	[progressView clear:self];
	return ( [[self getEmptyContainer] setString:""] );
}

- setProgressViewEmphasis
{
	int		pCount = [self paramCount];
	int		oldEmphasis;

	if ( pCount != 1 || !([[self getParam:1] validAsNumber]) ) {
		return ( [[self getEmptyContainer] setString:
			"Usage: setProgressViewEmphasis  <intValue>"] );
	}

	oldEmphasis = [progressView emphasis];
	[progressView setEmphasis:[[self getParam:1] intVal]];

	return ( [[self getEmptyContainer] setInt:oldEmphasis] );
}

- pp_version
{
	id		version;

	version = [[ self getEmptyContainer ] setString:"r" ];
	[ version appendString:PP_RELEASE ];
	[ version appendString:" v" ];
	[ version appendString:PP_VERSION ];
	[ version appendString:"    " ];
	[ version appendString:PP_RELTIME ];
	[ version appendString:"\n" ];
	[ version appendString:PP_COPYRIGHT ];

	return ( version );
}

// NOTE: Basically these buttons are not going to be useful until I can
// create a means for calling the XModule to check the event queue
// for a press on the button...  We will have to see how to do this.
// It is going to be rather tricky.

// Button target methods, with which the actions are setup in the progressPanel method.
//- pauseProgress:sender
//{
//	[self send:"pauseProgress" to:[self pageNamed:XM_ORD_THIS inStack:nil marked:NO]];
//	return ( self );
//}

//- cancelProgress:sender
//{
//	[self send:"cancelProgress" to:[self pageNamed:XM_ORD_THIS inStack:nil marked:NO]];
//	return ( self );
//}

@end

